import click


def upload_dataset_to_git(source, target):

    click.echo(
        f"git sync has not yet been implemented.\n"
        f"Please go to {source} and sync your git repository to {target} manually."
    )
